/*
 *  GOL - Game of life simulation software
 *  Copyright (C) 1996 Michael Raymond Brown
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Michael Raymond Brown
 *  EMail:  michael@arrakeen.demon.co.uk
 *  WWW:    http://www.arrakeen.demon.co.uk/
 *  Address:  Michael Brown
 *            Kia Ora
 *            Brook Street
 *            Great Bromley
 *            Nr Colchester
 *            Essex
 *            CO7 7HX
 *            England
 */


#ifdef __GNUC__
#pragma implementation
#endif

#include "VScrollArea.h"
#include <Xm/ScrollBar.h>
#include <iostream.h>

void xwVScrollArea::initScroll()
   {
    widget = XtVaCreateManagedWidget("drawarea"
		, xmDrawingAreaWidgetClass, scrollarea
		, 0);
    vscroll = XtVaCreateManagedWidget("scrollbar"
		, xmScrollBarWidgetClass, scrollarea
		, XmNorientation, XmVERTICAL
		, 0);
    hscroll = XtVaCreateManagedWidget("scrollbar"
		, xmScrollBarWidgetClass, scrollarea
		, XmNorientation, XmHORIZONTAL
		, 0);
    XmScrolledWindowSetAreas(scrollarea, hscroll, vscroll, widget);

    XtAddCallback(hscroll, XmNvalueChangedCallback, cbHScroll, (XtPointer)this);
    XtAddCallback(vscroll, XmNvalueChangedCallback, cbVScroll, (XtPointer)this);
    XtAddCallback(hscroll, XmNdragCallback, cbHScroll, (XtPointer)this);
    XtAddCallback(vscroll, XmNdragCallback, cbVScroll, (XtPointer)this);
    XtAddCallback(widget, XmNresizeCallback, cbResize, (XtPointer)this);

    oldX=0; oldY=0;
    init();

    setRange(1, 1);
   }
xwVScrollArea::xwVScrollArea(xwWidget* p, int x1, int y1, int x2, int y2)
   {parent = p;
    scrollarea = XtVaCreateManagedWidget("scrollbar"
		, xmScrolledWindowWidgetClass, parent->getWidget()
		, XmNscrollingPolicy, XmAPPLICATION_DEFINED

		, XmNleftAttachment,	XmATTACH_POSITION
		, XmNrightAttachment,	XmATTACH_POSITION
		, XmNtopAttachment,	XmATTACH_POSITION
		, XmNbottomAttachment,	XmATTACH_POSITION

		, XmNleftPosition,	x1
		, XmNrightPosition,	x2
		, XmNtopPosition,	y1
		, XmNbottomPosition,	y2

		, 0);
    initScroll();
   }
xwVScrollArea::xwVScrollArea(xwWidget* p)
   {parent = p;
    scrollarea = XtVaCreateManagedWidget("scrollbar"
		, xmScrolledWindowWidgetClass, parent->getWidget()
		, XmNscrollingPolicy, XmAPPLICATION_DEFINED
		, 0);
    initScroll();
   }
xwVScrollArea::~xwVScrollArea()
   {
    XtDestroyWidget(scrollarea);
    XtDestroyWidget(hscroll);
    XtDestroyWidget(vscroll);
   }


void xwVScrollArea::setOffsets(int x, int y)
   {oldX=x; oldY=y;
    XtVaSetValues(hscroll, XmNvalue, x, 0);
    XtVaSetValues(vscroll, XmNvalue, y, 0);
   }
void xwVScrollArea::getOffsets(int* x, int* y)
   {XtVaGetValues(hscroll, XmNvalue, x, 0);
    XtVaGetValues(vscroll, XmNvalue, y, 0);
   }

void xwVScrollArea::setRange(int width, int height)
   {Dimension w, h;
    XtVaGetValues(widget, XmNwidth,  &w, XmNheight, &h, 0);
    rangeX=width ; rangeY=height;

    XtVaSetValues(hscroll
		, XmNmaximum, rangeX
		, XmNincrement, 1
		, XmNpageIncrement, int(w>rangeX?1:w>1?(w/2):1)
		, XmNsliderSize, int(w>rangeX?rangeX:w>1?w:1)
		, 0);
    XtVaSetValues(vscroll
		, XmNmaximum, height
		, XmNincrement, 1
		, XmNpageIncrement, int(h>rangeY?1:h>1?(h/2):1)
		, XmNsliderSize, int(h>rangeY?rangeY:h>1?h:1)
		, 0);
   }
void xwVScrollArea::getRange(int* width, int* height)
   {*width =rangeX;
    *height=rangeY;
   }

void xwVScrollArea::getView(int* x, int* y, int* width, int* height)
   {int xl, yl;
    Dimension wl, hl;

    getOffsets(&xl, &yl); *x=xl; *y=yl;
    XtVaGetValues(widget, XmNwidth,  &wl, 0); *width  = wl;
    XtVaGetValues(widget, XmNheight, &hl, 0); *height = hl;
    if ((xl+ *width) > rangeX) *width =rangeX-xl;
    if ((yl+*height) > rangeY) *height=rangeY-yl;
   }



void xwVScrollArea::cbResize(Widget w, XtPointer client_data, XtPointer call_data)
   {xwVScrollArea* me = (xwVScrollArea*)client_data;
    me->setRange(me->rangeX, me->rangeY);
    if (me->handler) me->handler->onResize(me, me->handler_arg);
   }


void xwVScrollArea::cbHScroll(Widget w, XtPointer client_data, XtPointer call_data)
   {xwVScrollArea* me = (xwVScrollArea*)client_data;
    int width, height;
    me->getSize(&width, &height);
    if (me->handler) me->handler->onExpose(me, me->handler_arg, 0, 0, width, height);
   }

void xwVScrollArea::cbVScroll(Widget w, XtPointer client_data, XtPointer call_data)
   {xwVScrollArea* me = (xwVScrollArea*)client_data;
    int width, height;
    me->getSize(&width, &height);
    if (me->handler) me->handler->onExpose(me, me->handler_arg, 0, 0, width, height);
   }



